# -*- coding: utf-8 -*-
#
# The bottom half of the main window
#

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import os
import wx.lib.buttons as buttons
import editDialog
import utils

[
wxID_BOTTOM_WINDOW, wxID_AUTOPREVIEW, wxID_GO,
wxID_MENUEDIT, wxID_MENUREMOVE, wxID_MENUREMOVEANDPREVIEW,
wxID_THUMBSIZE, wxID_AUTOPREVIEW,
wxID_IMGPREVIEW, wxID_UNDOREDO,
] = [wx.NewId() for _init_coll_menuEdit_Items in range(10)]

# ------------ the bottom preview virtual listcrtl:
class ListCtrl(wx.ListCtrl):
    def __init__(self, parent, ID, pos=wx.DefaultPosition,
                 size=wx.DefaultSize):
        style = wx.LC_REPORT|wx.LC_VIRTUAL

        self.parent = parent.GetGrandParent()

        wx.ListCtrl.__init__(self, parent, ID, pos, size, style)

        self.InsertColumn(col=0, format=wx.LIST_FORMAT_LEFT,
              heading=_(u"Location"), width=225)
        self.InsertColumn(col=1, format=wx.LIST_FORMAT_LEFT,
              heading=_(u"Original Name"), width=245)
        self.InsertColumn(col=2, format=wx.LIST_FORMAT_LEFT,
              heading=_(u"New Name"), width=265)

        self.SetMinSize(wx.Size(-1,110))

        self.SetItemCount(0)
        self.showDirs = False
        self.mode = 'preview'

        self.green = wx.ListItemAttr()
        self.green.SetBackgroundColour('#97F27F')
        self.light_green = wx.ListItemAttr()
        self.light_green.SetBackgroundColour('#E5FFE5')
        self.red = wx.ListItemAttr()
        self.red.SetBackgroundColour('#FF1616')
        self.yellow = wx.ListItemAttr()
        self.yellow.SetBackgroundColour('#FDEB22')

        self.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.onRightClick)


    # right click menu
    def initMenu(self, menu):
        menu.edit = wx.MenuItem(menu, wxID_MENUEDIT, _(u"Manually Edit"))
        menu.edit.SetBitmap(wx.Bitmap(utils.iconPath(u'edit.png'),
            wx.BITMAP_TYPE_PNG))

        menu.remove = wx.MenuItem(menu, wxID_MENUREMOVE, _(u"Remove Item(s)"))
        menu.remove.SetBitmap(wx.Bitmap(utils.iconPath(u'errors.png'),
            wx.BITMAP_TYPE_PNG))

        menu.removeAndPreview = wx.MenuItem(menu, wxID_MENUREMOVEANDPREVIEW,
            _(u"Remove Item(s) and preview"))
        menu.removeAndPreview.SetBitmap(wx.Bitmap(utils.iconPath(u'errors.png'),
            wx.BITMAP_TYPE_PNG))

        menu.AppendItem(menu.edit)
        menu.AppendItem(menu.remove)
        menu.AppendItem(menu.removeAndPreview)
        #menu.AppendItem(parent.disable)

        self.Bind(wx.EVT_MENU, self.removeItems,
              id=wxID_MENUREMOVE)
        self.Bind(wx.EVT_MENU, self.removeItemsAndPreview,
              id=wxID_MENUREMOVEANDPREVIEW)
        self.Bind(wx.EVT_MENU, self.manualEdit,
              id=wxID_MENUEDIT)

    # Popup the menu
    def onRightClick(self, event):
        if hasattr(self.parent, 'toRename') and len(self.parent.toRename) != 0:
            self.rightClickMenu = wx.Menu()
            self.initMenu(self.rightClickMenu)
            self.PopupMenu(self.rightClickMenu)

    # remove selected items, needs renaming list adjusted afterwards
    def removeItems(self, event):
        itemNames = []
        item = -1
        while True:
            item = self.GetNextItem(item, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
            itemName = self.parent.toRename[item][0][0]
            if item == -1:
                break
            else:
                itemNames.append(itemName)
            self.parent.toRename.pop(item)
        self.parent.picker.removeItemsByName(itemNames)
        self.DeleteAllItems()
        self.SetItemCount(len(self.parent.toRename))

    def removeItemsAndPreview(self, event):
        self.removeItems(event)
        self.parent.onPreviewButton(False)

    def removeItemsAndCheckErrors(self, event):
        self.removeItems()
        self.parent.renamer.errorCheckItems()

    # allow editing single item
    def manualEdit(self, event):
        item = self.GetNextItem(-1, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)

        self.dlg = editDialog.Dialog(self, item)

        if self.dlg.ShowModal() == wx.ID_OK:
            self.setName(self.dlg.item, self.dlg.getValue())

    # set a new name manually
    def setName(self, item, new):
        if new:
            self.parent.toRename[item][1][0] = new
            self.parent.renamer.errorCheckItems()

    def setItemSelected(self, item):
        self.EnsureVisible(item)
        self.SetItemState(item, wx.LIST_STATE_SELECTED, wx.LIST_STATE_SELECTED)

    def setItemDeSelected(self, item):
        self.SetItemState(item, 0, wx.LIST_STATE_SELECTED|wx.LIST_STATE_FOCUSED)

    # get the info for an item from the list
    def getNames(self, item):
        item = self.parent.toRename[int(item)]
        original = item[0]
        renamed = item[1]
        return original, renamed

    def OnGetItemText(self, item, col):
        original, renamed = self.getNames(item)
        split = os.path.split(original[0])    
        if self.showDirs:
            to_rename = renamed[0].lstrip(os.path.sep)
        else:
            to_rename = os.path.split(renamed[0])[1]
        if col == 0:
            return split[0]
        elif col == 1:
            return split[1]
        else:
            return to_rename

    def OnGetItemImage(self, item):
        pickerList = self.parent.picker.Panel.ItemList
        original, renamed = self.getNames(item)
        if original[1]:
            # show preview if available
            if self.parent.prefs.get('showPreviewIcons') and\
              pickerList.thumbnails.has_key(original[0]):
                img = pickerList.thumbnails[original[0]]
            else:
                img = 1
        else:
            img = 0
        return img

    def getPreviewAttr(self, item):
        original, renamed = self.getNames(item)
        # make bad/warn names stand out
        if item in self.parent.bad:
            return self.red
        elif item in self.parent.warn:
            return self.yellow
        # make items that will be renamed stand out
        elif self.parent.prefs.get('showPreviewHighlight') and\
          (original[0] != renamed[0]):
            return self.light_green
        else:
            return None

    def getRenameAttr(self, item):
        original, renamed = self.getNames(item)
        if renamed[1] == False:
            return None
        elif renamed[1] and (original[0] != renamed[0]):
            return self.green
        elif renamed[1] == None:
            return self.red

    def OnGetItemAttr(self, item):
        if self.mode == 'preview':
            return self.getPreviewAttr(item)
        elif self.mode == 'rename':
            return self.getRenameAttr(item)



# ------------ the bottom panel of the main splitter
class MainPanel(wx.Panel):
    def sizer(self):
        buttonSizer = self.buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
        mainSizer = self.mainSizer = wx.BoxSizer(wx.VERTICAL)
        buttonSizer.AddWindow(self.preview,0,wx.LEFT, 10)
        buttonSizer.AddWindow(self.go,0,wx.LEFT, 10)
        buttonSizer.AddSpacer((-1,25),1)
        buttonSizer.AddWindow(self.imgPreview,0,wx.ALIGN_CENTER|wx.RIGHT,5)
        buttonSizer.AddWindow(self.thumbSize,0,wx.ALIGN_CENTER|wx.RIGHT,15)
        buttonSizer.AddWindow(self.autoPreview,0,wx.ALIGN_CENTER|wx.RIGHT,5)
        mainSizer.AddSizer(buttonSizer,0,wx.EXPAND|wx.ALIGN_LEFT|wx.TOP,5)
        mainSizer.AddWindow(self.display,1,wx.EXPAND|wx.ALL, 5)
        self.SetSizerAndFit(mainSizer)


    def initCtrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_BOTTOM_WINDOW, name=u'Panel',
                          parent=prnt, style=wx.TAB_TRAVERSAL)


        self.display = ListCtrl(self, -1, size=wx.Size(520, -1))

        #self.display.Bind(wx.EVT_LIST_ITEM_FOCUSED, self.main.onItemSelected,
        #                  self.display)

        self.display.Bind(wx.EVT_LIST_ITEM_SELECTED, self.main.onItemSelected,
                          self.display)

        self.preview = buttons.GenBitmapTextButton(self, wxID_AUTOPREVIEW,
                       wx.Bitmap(utils.iconPath(u'preview.png'),
                       wx.BITMAP_TYPE_PNG),_(u"Preview"), size=(-1, 26))
        self.Bind(wx.EVT_BUTTON, self.main.onPreviewButton,
                  id=wxID_AUTOPREVIEW)

        self.go = buttons.GenBitmapTextButton(self, wxID_GO,
                  wx.Bitmap(utils.iconPath(u'go.png'),wx.BITMAP_TYPE_PNG),
                  _(u"Go!"), size=(-1, 26))
        self.go.Enable(False)
        self.go.Bind(wx.EVT_BUTTON, self.main.renameItems, id=wxID_GO)

        self.imgPreview = wx.CheckBox(id=wxID_IMGPREVIEW,
          label=_(u"Show image thumbnails"), name=u'imgPreview', parent=self,
          style=0)
        self.imgPreview.SetToolTipString(_(u"Can slow preview considerably"))
        self.imgPreview.SetValue(False)
        self.imgPreview.Bind(wx.EVT_CHECKBOX, self.refreshPicker)

        self.thumbSize = wx.Choice(id=wxID_THUMBSIZE,
                         choices=[u'32',u'64',u'128',u'256'],
                         name=u'dirsPlace', parent=self, size=wx.Size(62,-1),
                         style=0)
        self.thumbSize.SetSelection(1)
        self.thumbSize.SetToolTipString(_(u"Thumbnail Size"))
        self.thumbSize.Bind(wx.EVT_CHOICE, self.setThumbnailSize)

        if not utils.pilIsLoaded():
            self.imgPreview.SetValue(False)
            self.imgPreview.Enable(False)
            self.thumbSize.Enable(False)


        self.autoPreview = wx.CheckBox(id=wxID_AUTOPREVIEW,
          label=_(u"Automatic Preview"), name=u'autoPreview', parent=self,
          style=0)
        self.autoPreview.SetToolTipString(_(u"Disable when working with many items"))
        self.autoPreview.SetValue(True)


    def __init__(self, prnt, MainWindow):
        self.main = MainWindow
        self.initCtrls(prnt)
        self.sizer()
        self.createUndoRedo(_("Undo"))
        # disable if no undo files exist (to lessen user confusion)
        if not os.path.exists(utils.stdPath(u'undo/original.bak')) or not\
          os.path.exists(utils.stdPath(u'undo/renamed.bak')):
            self.undoRedo.Enable(False)

    def refreshPicker(self, event):
        self.main.picker.refresh(event)

    # only re-preview if image preview is enabled and size has changed
    def setThumbnailSize(self, event):
        if hasattr(self.main.picker.Panel.ItemList, 'thumbSize'):
            thumbSize = self.main.picker.Panel.ItemList.thumbSize[0]
        else:
            thumbSize = int(event.GetString())
        if self.imgPreview.GetValue() and int(event.GetString()) != thumbSize:
            self.refreshPicker(event)

    # deletes and creates a new button, hack to change image
    def createUndoRedo(self,label):
        if label == _("Undo"):
            img = wx.Bitmap(utils.iconPath(u'undo.ico'),wx.BITMAP_TYPE_ICO)
        else:
            img = wx.Bitmap(utils.iconPath(u'redo.ico'),wx.BITMAP_TYPE_ICO)

        if hasattr(self, 'undoRedo'):
            self.undoRedo.Destroy()

        self.undoRedo = buttons.GenBitmapTextButton(self, wxID_UNDOREDO,img, label, size=(-1, 26))
        # do not use wx.EVT_BUTTON, causes seg fault
        self.undoRedo.Bind(wx.EVT_LEFT_DOWN, self.main.undoRename, id=wxID_UNDOREDO)
        self.buttonSizer.Insert(2,self.undoRedo,proportion=0,flag=wx.LEFT,border=25)
        self.Layout()

    def setUndoRedoType(self,op):
        if op == 'check':
            if self.undoRedo.GetLabel() == _("Undo"):
                self.createUndoRedo(_("Redo"))
            else:
                self.createUndoRedo(_("Undo"))
        elif op == 'undo':
            self.createUndoRedo(_("Undo"))