# -*- coding: utf-8 -*-
#
"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import utils
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin

[wxID_ERRORPANEL, wxID_ERRORPANELERRORSLIST, wxID_ERRORPANELERRORSSTATICTEXT1,
 wxID_ERRORPANELERRORSREMOVEERRORS, wxID_ERRORPANELERRORSREMOVEWARNINGS,
 wxID_ERRORPANELERRORSSTATICTEXT2, wxID_ERRORPANELERRORSREMOVE,
] = [wx.NewId() for initCtrls in range(7)]

class listCtrl(wx.ListCtrl, ListCtrlAutoWidthMixin):
    def __init__(self, parent, ID, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0):
        wx.ListCtrl.__init__(self, parent, ID, pos, size, style)
        # causes freeze under winXP
        #ListCtrlAutoWidthMixin.__init__(self)
        self.InsertColumn(col=1, format=wx.LIST_FORMAT_LEFT, heading=_(u"Error"),
              width=260)
        self.InsertColumn(col=2, format=wx.LIST_FORMAT_LEFT, heading=_(u"File"),
              width=350)


class Panel(wx.Panel):
    def sizer(self):
    	mainSizer = wx.BoxSizer(wx.VERTICAL)
    	options = wx.BoxSizer(wx.HORIZONTAL)
    	options.Add(self.remove,0,wx.LEFT,225)
    	options.Add(self.staticText1,0,wx.ALIGN_CENTER|wx.LEFT,5)
    	options.Add(self.removeWarnings,0,wx.ALIGN_CENTER|wx.LEFT,5)
    	options.Add(self.removeErrors,0,wx.ALIGN_CENTER|wx.LEFT,5)
    	options.Add(self.staticText2,0,wx.ALIGN_CENTER|wx.LEFT,10)

        mainSizer.Add(self.errorsList,1,wx.EXPAND|wx.TOP|wx.BOTTOM,3)
        mainSizer.Add(options,0,wx.BOTTOM,3)

        self.SetSizerAndFit(mainSizer)


    def initCtrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_ERRORPANEL, name=u'errorPanel',
              parent=prnt, pos=wx.Point(295, 251), size=wx.Size(650, 302),
              style=wx.TAB_TRAVERSAL)
        self.SetClientSize(wx.Size(642, 273))

        self.errorsList = listCtrl(self, wxID_ERRORPANELERRORSLIST,
              size=wx.Size(-1, -1), style=wx.LC_REPORT)
        self.errorsList.Bind(wx.EVT_LIST_ITEM_SELECTED, self.onActivateError,
              id=wxID_ERRORPANELERRORSLIST)

        self.staticText1 = wx.StaticText(id=wxID_ERRORPANELERRORSSTATICTEXT1,
              label=_(u"items with:"), name=u'staticText1', parent=self, style=0)

        self.remove = wx.Button(id=wxID_ERRORPANELERRORSREMOVE, name=u'remove',
              parent=self, label=_(u"Remove"), style=wx.BU_EXACTFIT)
        self.remove.SetToolTipString(_(u"Remove items from renaming selection"))
        self.remove.Bind(wx.EVT_BUTTON, self.removeNamesByType)

        self.removeWarnings = wx.CheckBox(id=wxID_ERRORPANELERRORSREMOVEWARNINGS, label=_(u"warnings"),
              name=u'removeWarnings', parent=self, style=0)
        self.removeWarnings.SetValue(False)
        self.removeWarnings.SetToolTipString(_(u"all items with warnings"))

        self.removeErrors = wx.CheckBox(id=wxID_ERRORPANELERRORSREMOVEERRORS, label=_(u"errors"),
              name=u'removeErrors', parent=self, style=0)
        self.removeErrors.SetValue(True)
        self.removeErrors.SetToolTipString(_(u"all items with errors"))

        self.staticText2 = wx.StaticText(id=wxID_ERRORPANELERRORSSTATICTEXT2,
              label=_("from above items."), name=u'staticText1', parent=self, style=0)


    def __init__(self, Core, parent, main_window):
    	global main
        main = main_window

        self.initCtrls(parent)
        # adjust button sizes
        utils.adjustExactbuttons(self)
        self.sizer()

        # to store for removal
        self.errors = []
        self.warnings = []

        self.imgs = wx.ImageList(16, 16)
        self.warn = self.imgs.Add(wx.Bitmap(utils.iconPath(u'warn.ico'),wx.BITMAP_TYPE_ICO))
        self.bad = self.imgs.Add(wx.Bitmap(utils.iconPath(u'failed.ico'),wx.BITMAP_TYPE_ICO))
        self.errorsList.AssignImageList(self.imgs, wx.IMAGE_LIST_SMALL)


    def displayErrors(self, errorLog, warn, bad):
        errImgs = {u'warn':self.warn, u'bad':self.bad}
        badCount = 0
        warnCount = 0
        for error in errorLog:
            if not (error[0] in warn and error[0] in bad) or error[3] == u'bad':
                if error[3] == 'bad':
                    badCount += 1
                    self.errors.append(error[1])
                else:
                    warnCount += 1
                    self.warnings.append(error[1])
                index = self.errorsList.InsertImageStringItem(0, u" "+error[2], errImgs[error[3]])
                self.errorsList.SetStringItem(index, 1, unicode(error[1]))
                self.errorsList.SetItemData(index, error[0])

        self.errorsList.SetColumnWidth(0, -1)
        self.errorsList.SetColumnWidth(1, -1)
        main.notebook.SetPageText(3,_(u"Errors: %s - Warnings: %s")%(badCount, warnCount))

        if len(warn) != 0 and len(bad) == 0:
            main.setStatusMsg(_(u"Ready to rename %s items, but with %s warnings")%(len(main.toRename),warnCount),u'warn')
        else:
            main.setStatusMsg(_(u"%s total items, %s have problems") %(len(main.toRename), badCount+warnCount),u'failed')

    def clearErrors(self):
        self.errors = []
        self.warnings = []
        self.errorsList.DeleteAllItems()
        main.notebook.SetPageText(3,_(u"Errors/Warnings: 0"))

    def onActivateError(self, event):
        currentItem = event.m_itemIndex
        item_numb = self.errorsList.GetItemData(currentItem)
        main.bottomWindow.display.EnsureVisible(item_numb)

    # remove errors from renaming selection
    def removeNamesByType(self, event):
        if self.removeErrors.GetValue():
            self.removeNames(self.errors)
        if self.removeWarnings.GetValue():
            self.removeNames(self.warnings)
        main.picker.enableButtons()
        main.showPreview(event)

    def removeNames(self, errors):
        main.picker.removeItemsByName(errors)
        errors = []


