# -*- coding: utf-8 -*-

# wxWidgets picker panel and associated classes.
# Only responsible for views, All interactions must be handled by picker Core

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

import wx
import utils
import os
try:
    import Image
except:
    utils.makeErrMsg(_("Python Imaging Library (PIL) not found.\nRefer to readme file for install instructions.\n\nYou will not be able to preview images."),
                     _("PIL needed"))


[wxID_PICKERPANEL, wxID_PICKERPANELBROWSE, wxID_SELECTIONAREADIRPICKER,
 wxID_PICKERPANELDIVIDER1, wxID_PICKERPANELFILESON,
 wxID_PICKERPANELFILTERSEL, wxID_PICKERPANELFOLDERSON, wxID_PICKERPANELNOT_TYPE,
 wxID_PICKERPANELOK, wxID_PICKERPANELPATH, wxID_PICKERPANELPATH_HELP,
 wxID_SELECTIONAREAFPICKER, wxID_PICKERPANELSELECT, wxID_PICKERPANELSELECT_ALL,
 wxID_PICKERPANELSELECT_NONE, wxID_PICKERPANELSTATICTEXT1,
 wxID_PICKERPANELWALKIT, wxID_PICKERPANELFILTERBYRE, wxID_SELECTIONAREA,
 wxID_PICKERPANELIGNORECASE, wxID_PICKERPANELUSELOCALE, wxID_PICKERPANELWALKDEPTH,
 wxID_PICKERPANELSTATICTEXT2,
] = [wx.NewId() for initCtrls in range(23)]


# Panel refers to instance as dirpicker
class DirectoryList(wx.GenericDirCtrl):
    def __init__(self, parent, ID, name, showHidden):
        self.parent = parent.GetParent()
        wx.GenericDirCtrl.__init__(self, parent, ID, name,
          defaultFilter=0, filter='', style=wx.DIRCTRL_DIR_ONLY)

        if showHidden:
            self.ShowHidden(True)

        isz = (16,16)
        il = wx.ImageList(isz[0], isz[1])
        # closed folder:
        il.Add(wx.Bitmap(utils.iconPath(u'folder16.png'),wx.BITMAP_TYPE_PNG))
        # open folder:
        il.Add(wx.Bitmap(utils.iconPath(u'folderOpen.png'),wx.BITMAP_TYPE_PNG))
        # root of filesystem (linux):
        il.Add(wx.Bitmap(utils.iconPath(u'root.ico'),wx.BITMAP_TYPE_ICO))
        # drive letter (windows):
        il.Add(wx.Bitmap(utils.iconPath(u'hdd.ico'),wx.BITMAP_TYPE_ICO))
        # cdrom drive:
        il.Add(wx.Bitmap(utils.iconPath(u'cdrom.ico'),wx.BITMAP_TYPE_ICO))
        # removable drive on win98:
        il.Add(wx.Bitmap(utils.iconPath(u'fdd.ico'),wx.BITMAP_TYPE_ICO))
        # removable drive (floppy, flash, etc):
        il.Add(wx.Bitmap(utils.iconPath(u'fdd.ico'),wx.BITMAP_TYPE_ICO))
        self.il = il
        self.GetTreeCtrl().SetImageList(il)
        self.bind()

    def dirChange(self,event):
        self.parent.path.SetValue(self.GetPath())
        self.parent.refreshItems(event)

    def unbind(self):
        self.Bind(wx.EVT_TREE_SEL_CHANGED, utils.devNull)

    def bind(self):
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.dirChange)



# picker list and associated functions
class ItemList(wx.ListCtrl):
    def __init__(self, parent, id, name, MainWindow):
        self.pickerPanel = parent.GetParent()

        global main
        main = MainWindow

        style = wx.LC_LIST
        #style = wx.LC_REPORT|wx.LC_NO_HEADER|wx.LC_VIRTUAL

        wx.ListCtrl.__init__(self, parent, id=id, name=name, style=style)
        self.thumbnails = {}
        self.createImageList(main.bottomWindow.imgPreview.GetValue())

        """
        self.SetItemCount(100000)
        self.InsertColumn(0, "First")
        self.SetColumnWidth(0, 470)

        self.attr1 = wx.ListItemAttr()
        self.attr1.SetBackgroundColour("yellow")

        self.attr2 = wx.ListItemAttr()
        self.attr2.SetBackgroundColour("light blue")

        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.OnItemDeselected)
        """
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.onItemSelected)
        self.Bind(wx.EVT_KEY_DOWN, self.onKeyDown)
        self.Bind(wx.EVT_KEY_UP, self.onKeyUp)

        # keep shift key value
        self.shiftDown = False
        self.totalSelected = 0


    def createImageList(self, imgPreview):
        # get thumbnail size
        if imgPreview:
            xy = int(main.bottomWindow.thumbSize.GetStringSelection())
            thumbSize=(xy,xy)
        else:
            thumbSize=(16,16)
        # apply to list
        self.imgs = wx.ImageList(thumbSize[0],thumbSize[1])
        self.folderIco = self.imgs.Add(wx.Bitmap(utils.iconPath(u'folder%s.png'%thumbSize[0]),
           wx.BITMAP_TYPE_PNG))
        self.fileIco = self.imgs.Add(wx.Bitmap(utils.iconPath(u'file%s.png'%thumbSize[0]),
           wx.BITMAP_TYPE_PNG))
        self.SetImageList(self.imgs, wx.IMAGE_LIST_SMALL)
        return thumbSize


    # gets item info and parses it for inclusion in renaming list
    def getItemInfo(self, item):
        item_txt = self.GetItemText(item)
        item_txt = item_txt.lstrip(os.sep)
        fullItem = os.path.join(self.pickerPanel.path.GetValue(),item_txt)
        IsFile = os.path.isfile(fullItem)
        fullItem = [fullItem,IsFile]
        return fullItem


    # Need to know if shift is pressed to adjust selection
    def onKeyDown(self, event):
        if event.m_keyCode == wx.WXK_SHIFT:
            self.shiftDown = True
        else:
            self.shiftDown = False


    def onKeyUp(self, event):
        self.shiftDown = False
        if event.m_keyCode == wx.WXK_SHIFT:
            self.totalSelected = 0
            self.pickerPanel.enableButtons()
            main.showPreview(True)


    # show selected items and add/remove from renaming list
    def onItemSelected(self, event):
        currentItem = event.m_itemIndex
        item = wx.ListItem()
        item.SetId(currentItem)
        fullItem = self.getItemInfo(currentItem)

        # avoid selecting same item twice when using shift key
        if self.shiftDown:
            self.totalSelected += 1

        if self.totalSelected != 1:
            # add to list / show selected:
            if fullItem not in self.pickerPanel.Core.joinedItems:
                item.SetBackgroundColour(utils.HighlightClr)
                item.SetTextColour(utils.HighlightTxtClr)
                self.pickerPanel.Core.joinedItems.append(fullItem)
            # remove from list / show deselected:
            else:
                item.SetBackgroundColour(utils.BackgroundClr)
                item.SetTextColour(utils.TxtClr)
                self.pickerPanel.Core.joinedItems.remove(fullItem)
            # apply highlighting
            self.SetItem(item)

        if len(main.renamer.operations) == 0:
            msg = _(u"%s items were (de)selected") % (self.totalSelected+1)
            main.setStatusMsg(msg, 'eyes')

        # Deselect item to avoid user confusion
        main.currentItem = None
        self.Select(currentItem, False)

        if not self.shiftDown:
            self.pickerPanel.enableButtons()
            main.showPreview(True)


    # create thumbnail from image file
    def imageToPil(self, source, thumbSize):
        source = Image.open(source, 'r')
        source.thumbnail(thumbSize, Image.ANTIALIAS)
        image = apply(wx.EmptyImage, source.size)
        image.SetData(source.convert("RGB").tostring())
        return wx.BitmapFromImage(image)


    # add items found to list
    def addItems(self, root, folders, files):
        self.thumbnails = {}
        supported = ('.png','.jpg','.bmp','.tif','.jpeg',
                     '.tiff','.gif','.ico','.thm')
        imgPreview = main.bottomWindow.imgPreview.GetValue()
        thumbSize = self.thumbSize = self.createImageList(imgPreview)

        if imgPreview and utils.pilIsLoaded():
            imgPreview = True
            def getExt(item):
                return os.path.splitext(item)[1].lower()
        else:
            imgPreview = False

        # list folders first
        folders.sort(key=lambda x: x.lower())
        for i in range(len(folders)):
            self.InsertImageStringItem(i, folders[i], 0)

        # ... then files
        def makeKey(f): return (os.path.dirname(f), f.lower())
        files.sort(key=makeKey)

        i = len(folders)
        for item in files:
            # preview images
            if imgPreview is True and getExt(item) in supported:
                fullitem = os.path.join(root,item.lstrip('/'))
                try:
                    thumb = self.imageToPil(fullitem,thumbSize)

                except IOError:
                    img = 1
                else:
                    img = self.imgs.Add(thumb)
                    # link name with number in imageList for main window
                    self.thumbnails[fullitem] = img
            else:
                img = 1
            self.InsertImageStringItem(i, item, img)
            i += 1

    """
    def OnItemActivated(self, event):
        self.currentItem = event.m_itemIndex
        self.log.WriteText("OnItemActivated: %s\nTopItem: %s\n" %
                           (self.GetItemText(self.currentItem), self.GetTopItem()))


    def OnItemDeselected(self, evt):
        print self.items[evt.m_itemIndex]
        #print "OnItemDeselected: %s" % evt.m_itemIndex

    #---------------------------------------------------
    # These methods are callbacks for implementing the
    # "virtualness" of the list...  Normally you would
    # determine the text, attributes and/or image based
    # on values from some external data source, but for
    # this demo we'll just calculate them
    def OnGetItemText(self, item, col):
        return "Item %d, column %s" % (item, col)

    def OnGetItemImage(self, item):
        return self.folderIco


    def OnGetItemAttr(self, item):
        if item % 3 == 1:
            return self.attr1
        elif item % 3 == 2:
            return self.attr2
        else:
            return None
    """


class Panel(wx.Panel):
    def sizer(self):
        # adjust button sizes
        #utils.adjustExactbuttons(self,ignore=('OK'))
        utils.adjustExactbuttons(self)

        PathSizer = self.PathSizer = wx.BoxSizer(orient=wx.HORIZONTAL)
        SelectSizer = self.SelectSizer = wx.BoxSizer(orient=wx.HORIZONTAL)
        mainSizer = self.mainSizer = wx.BoxSizer(orient=wx.VERTICAL)

        PathSizer.AddWindow(self.BROWSE,0,wx.ALL, 5, wx.ALIGN_CENTER)
        PathSizer.AddWindow(self.path,1,wx.ALIGN_CENTER|wx.RIGHT, 5)
        PathSizer.AddWindow(self.walkIt,0,wx.ALIGN_CENTER|wx.RIGHT, 5)
        PathSizer.AddWindow(self.staticText2,0,wx.ALIGN_CENTER|wx.RIGHT, 3)
        PathSizer.AddWindow(self.walkDepth,0,wx.ALIGN_CENTER|wx.RIGHT, 15)
        PathSizer.AddWindow(self.OK,0,wx.RIGHT|wx.ALIGN_CENTER,8)

        SelectSizer.AddWindow(self.notType,0,wx.ALIGN_CENTER)
        SelectSizer.AddWindow(self.staticText1,0,wx.ALIGN_CENTER|wx.LEFT,3)
        SelectSizer.AddWindow(self.FilterSel,20,wx.ALIGN_CENTER)
        SelectSizer.AddWindow(self.filterByRE,0,wx.ALIGN_CENTER|wx.LEFT,5)
        SelectSizer.AddWindow(self.ignoreCase,0,wx.ALIGN_CENTER|wx.LEFT,2)
        SelectSizer.AddWindow(self.useLocale,0,wx.ALIGN_CENTER|wx.LEFT,2)
        SelectSizer.AddSpacer((-1,-1),1,wx.EXPAND)
        SelectSizer.AddWindow(self.divider1,0,wx.ALIGN_CENTER)
        SelectSizer.AddSpacer((-1,-1),1,wx.EXPAND)
        SelectSizer.AddWindow(self.select,0,wx.RIGHT|wx.ALIGN_CENTER,5)
        SelectSizer.AddWindow(self.foldersOn,0,wx.ALIGN_CENTER|wx.RIGHT,7)
        SelectSizer.AddWindow(self.filesOn,0,wx.ALIGN_CENTER|wx.RIGHT,5)
        SelectSizer.AddSpacer((-1,-1),1,wx.EXPAND)
        SelectSizer.AddWindow(self.selectAll,0,wx.RIGHT,2)
        SelectSizer.AddWindow(self.selectNone,0)
        SelectSizer.AddSpacer((-1,-1),1,wx.EXPAND)

        mainSizer.AddWindow(self.pathHelp,0,wx.EXPAND|wx.LEFT|wx.TOP, 6)
        mainSizer.AddSizer(PathSizer,0,wx.EXPAND|wx.ALIGN_CENTER)
        mainSizer.AddSizer(SelectSizer,0,wx.LEFT|wx.EXPAND,6)
        mainSizer.AddWindow(self.selectionArea,1,wx.EXPAND|wx.TOP, 3)

        self.SetSizerAndFit(mainSizer)

    def initCtrls(self, prnt):
        wx.Panel.__init__(self, id=wxID_PICKERPANEL, name=u'pickerPanel',
              parent=prnt, style=wx.NO_BORDER)

        self.path = wx.TextCtrl(id=wxID_PICKERPANELPATH, name=u'path',
              parent=self, style=wx.TE_PROCESS_ENTER, value='')
        self.path.Bind(wx.EVT_TEXT_ENTER, self.Core.setPanelPath,
              id=wxID_PICKERPANELPATH)

        self.OK = wx.Button(id=wxID_PICKERPANELOK, label=_(u"Refresh"), name=u'OK',
              parent=self, style=wx.BU_EXACTFIT)
        #reloadImg = wx.Bitmap(utils.iconPath(u'reload.png'), wx.BITMAP_TYPE_PNG)
        #self.OK = wx.BitmapButton(bitmap=reloadImg, id=wxID_PICKERPANELOK, name=u'OK',
        #      parent=self, style=wx.BU_AUTODRAW)
        self.OK.Enable(True)
        self.OK.SetToolTipString(_(u"Load or reload current path"))
        self.OK.Bind(wx.EVT_BUTTON, self.Core.setPanelPath, id=wxID_PICKERPANELOK)

        self.BROWSE = wx.Button(id=wxID_PICKERPANELBROWSE, label=_(u"Browse"),
          name=u'BROWSE', parent=self, style=wx.BU_EXACTFIT)
        self.BROWSE.SetToolTipString(_(u"Browse for path"))
        self.BROWSE.Bind(wx.EVT_BUTTON, self.onBrowseButton,
          id=wxID_PICKERPANELBROWSE)

        self.pathHelp = wx.StaticText(id=wxID_PICKERPANELPATH_HELP,
              label=_(u"Input/Paste path and press OK, or BROWSE for path:"),
              name=u'pathHelp', parent=self, style=0)

        self.select = wx.StaticText(id=wxID_PICKERPANELSELECT, label=_(u"Select:"),
              name=u'select', parent=self, style=0)

        self.selectAll = wx.Button(id=wxID_PICKERPANELSELECT_ALL, label=_(u"all"),
              name=u'selectAll', parent=self, style=wx.BU_EXACTFIT)
        self.selectAll.Enable(False)
        self.selectAll.Bind(wx.EVT_BUTTON, self.onSelectAll,
              id=wxID_PICKERPANELSELECT_ALL)

        self.selectNone = wx.Button(id=wxID_PICKERPANELSELECT_NONE,
              label=_(u"none"), name=u'selectNone', parent=self, style=wx.BU_EXACTFIT)
        self.selectNone.Enable(False)
        self.selectNone.Bind(wx.EVT_BUTTON, self.onSelectNone,
              id=wxID_PICKERPANELSELECT_NONE)

        self.foldersOn = wx.CheckBox(id=wxID_PICKERPANELFOLDERSON, label=_(u"Folders"),
              name=u'foldersOn', parent=self, style=0,)
        self.foldersOn.SetValue(True)
        self.foldersOn.Bind(wx.EVT_CHECKBOX, self.refreshItems,
              id=wxID_PICKERPANELFOLDERSON)

        self.filesOn = wx.CheckBox(id=wxID_PICKERPANELFILESON, label=_(u"Files"),
              name=u'filesOn', parent=self, style=0)
        self.filesOn.SetValue(True)
        self.filesOn.Bind(wx.EVT_CHECKBOX, self.refreshItems,
              id=wxID_PICKERPANELFILESON)

        keys = self.getInSearches()
        self.FilterSel = wx.ComboBox(choices=keys, id=wxID_PICKERPANELFILTERSEL,
              name=u'FilterSel', parent=self, style=wx.TE_PROCESS_ENTER)
        self.FilterSel.SetSelection(0)
        self.FilterSel.SetToolTipString(_(u"Names containing (Use menu or enter text)"))
        self.FilterSel.Bind(wx.EVT_COMBOBOX, self.onFilterSel,
              id=wxID_PICKERPANELFILTERSEL)
        self.FilterSel.Bind(wx.EVT_TEXT_ENTER, self.refreshItems,
              id=wxID_PICKERPANELFILTERSEL)

        self.staticText1 = wx.StaticText(id=wxID_PICKERPANELSTATICTEXT1,
              label=_(u"Contaning:"), name=u'staticText1', parent=self, style=0)

        self.filterByRE = wx.CheckBox(id=wxID_PICKERPANELFILTERBYRE, label=_(u"Reg-Expr"),
              name=u'filterByRE', parent=self, style=0)
        self.filterByRE.SetValue(False)
        self.filterByRE.SetToolTipString(_(u"Evaluate filter as a regular expression"))
        self.filterByRE.Bind(wx.EVT_CHECKBOX, self.regexOptions,
              id=wxID_PICKERPANELFILTERBYRE)

        self.ignoreCase = wx.CheckBox(id=wxID_PICKERPANELIGNORECASE, label=_(u"I"),
              name=u'ignoreCase', parent=self, style=0)
        self.ignoreCase.SetValue(True)
        self.ignoreCase.Enable(False)
        self.ignoreCase.SetToolTipString(_(u"case-Insensitive match"))
        self.ignoreCase.Bind(wx.EVT_CHECKBOX, self.refreshItems,
              id=wxID_PICKERPANELIGNORECASE)

        self.useLocale = wx.CheckBox(id=wxID_PICKERPANELUSELOCALE, label=_(u"U"),
              name=u'useLocale', parent=self, style=0)
        self.useLocale.SetValue(True)
        self.useLocale.Enable(False)
        self.useLocale.SetToolTipString(_(u"Unicode match (\w matches 'a','b','c', etc)"))
        self.useLocale.Bind(wx.EVT_CHECKBOX, self.refreshItems,
              id=wxID_PICKERPANELUSELOCALE)

        self.walkIt = wx.CheckBox(id=wxID_PICKERPANELWALKIT, label=_(u"Recursive"),
              name=u'walkIt', parent=self, style=0)
        self.walkIt.SetValue(False)
        self.walkIt.SetToolTipString(_(u"Get all files in directory and sub-directories, but no folders"))
        self.walkIt.Bind(wx.EVT_CHECKBOX, self.onRecursiveCheckbox,
              id=wxID_PICKERPANELWALKIT)

        self.staticText2 = wx.StaticText(id=wxID_PICKERPANELSTATICTEXT2,
              label=_(u"depth:"), name=u'staticText2', parent=self, style=0)
        self.staticText2.Enable(False)

        self.walkDepth = wx.SpinCtrl(id=wxID_PICKERPANELWALKDEPTH, initial=0,
              max=999, min=0, name=u'walkDepth', parent=self,
              size=wx.Size(56, -1), style=wx.SP_ARROW_KEYS, value='0')
        self.walkDepth.SetValue(0)
        self.walkDepth.SetToolTipString(_(u"Number of levels to descend, 0 = unlimited"))
        self.walkDepth.Enable(False)

        self.notType = wx.CheckBox(id=wxID_PICKERPANELNOT_TYPE, label=_(u"Not"),
              name=u'notType', parent=self, style=0)
        self.notType.SetValue(False)
        self.notType.SetToolTipString(_(u"NOT containing"))
        self.notType.Bind(wx.EVT_CHECKBOX, self.refreshItems,
              id=wxID_PICKERPANELNOT_TYPE)

        self.divider1 = wx.StaticLine(id=wxID_PICKERPANELDIVIDER1,
              name=u'divider1', parent=self, size=wx.Size(3, 22), style=wx.LI_VERTICAL)

        # create split window:
        self.selectionArea = wx.SplitterWindow(parent=self, id=wxID_SELECTIONAREA,
          name=u'selectionArea', style=wx.SP_LIVE_UPDATE)

        # directory picker:
        if main.prefs.get(u'useDirTree'):
            self.createDirTree()

        # file picker:
        self.ItemList = ItemList(id=wxID_SELECTIONAREAFPICKER, name=u'picker',
                                 parent=self.selectionArea,MainWindow=main)

        self.initSplitter()


    def __init__(self, Core, parent, MainWindow):
        global main
        main = MainWindow
        self.Core = Core

        # build GUI
        self.initCtrls(parent)
        self.sizer()


    def refreshItems(self, event):
        self.Core.refresh(True)


    # If a key is pecified, get the requested value,
    # otherwise return all keys
    def getInSearches(self, key=False):
        if key != False:
            value = False
            for search in self.Core.CommonSearches:
                if search[0] == key:
                    value = search[1]
                    break
            return value
        else:
            keys = []
            for search in self.Core.CommonSearches:
                keys.append(search[0])
            return keys

    # Adjust after loading config
    def onConfigLoad(self,):
        self.regexOptions(False)
    

    # Used when a config is loaded to ensure dirpicker behaves correctly
    def resetDirPickerOnConfigLoad(self):
        if main.prefs.get('useDirTree') and \
        self.dirPicker.GetPath() != self.path.GetValue():
            self.dirPicker.unbind()
            self.dirPicker.SetPath(self.path.GetValue())
            self.dirPicker.bind()
        main.picker.refresh(True)


    # Create the directory tree:
    def createDirTree(self):
        self.dirPicker = DirectoryList(
                self.selectionArea,
                wxID_SELECTIONAREADIRPICKER,
                u'dirPicker',
                main.prefs.get('showHiddenDirs')
            )
        self.dirPicker.bind()


    # Initialise split window based on preferences
    def initSplitter(self):
        if main.prefs.get('useDirTree'):
            self.selectionArea.SplitVertically(self.dirPicker, self.ItemList, 280)
        else:
            self.selectionArea.Initialize(self.ItemList)


    # Hide or show directory tree:
    def setTree(self):
        if main.prefs.get(u'useDirTree'):
            self.createDirTree()
            self.dirPicker.SetPath(self.path.GetValue())
        else:
            self.selectionArea.Unsplit(self.dirPicker)
            self.dirPicker.Destroy()
        self.initSplitter()


    # Get and set path:
    def onBrowseButton(self, event):
        dlg = wx.DirDialog(self,_(u"Choose a directory:"),
            style=wx.DD_DEFAULT_STYLE)
        dlg.SetPath(self.path.GetValue())
        try:
            if dlg.ShowModal() == wx.ID_OK:
                dir = dlg.GetPath()
                self.path.SetValue(dir)
                self.Core.setPanelPath(event)
        finally: dlg.Destroy()


    # Set path:
    def setPath(self, event):
        if main.prefs.get(u'useDirTree'):
            # keep a record of the original path
            path = self.path.GetValue()
            # need to temporarily unbind to avoid showing picker twice
            self.dirPicker.Bind(wx.EVT_TREE_SEL_CHANGED, utils.devNull)
            try:
                self.dirPicker.SetPath(path)
            except AttributeError:
                pass
            # OK bind again
            self.dirPicker.Bind(wx.EVT_TREE_SEL_CHANGED, self.dirPicker.dirChange)
            # if tree won't display original path, force it
            if self.dirPicker.GetPath() != path:
                self.path.SetValue(path)


    # Set common searches
    def onFilterSel(self, event):
        filter = self.FilterSel
        if self.getInSearches(filter.GetStringSelection()):
            checkboxes = (self.filterByRE, self.ignoreCase, self.useLocale)
            for chckbx in checkboxes:
                chckbx.SetValue(True)
                chckbx.Enable(True)
        else:
            filter.SetLabel(u'')
            filter.SetValue(u'')
        self.refreshItems(event)


    # Add all items to renaming list
    def onSelectAll(self, event):
        # clear list
        self.Core.clearJoinedItems()
        for i in range(self.ItemList.GetItemCount()):
            # add to list
            self.Core.joinedItems.append(self.ItemList.getItemInfo(i))
            # show selected in picker
            item = wx.ListItem()
            item.SetId(i)
            #item.SetState(wx.LIST_STATE_SELECTED|wx.LIST_STATE_FOCUSED)
            item.SetBackgroundColour(utils.HighlightClr)
            item.SetTextColour(utils.HighlightTxtClr)
            self.ItemList.SetItem(item)
        # enable buttons
        self.ItemList.Refresh()
        self.selectNone.Enable(True)
        main.menuPicker.getNoneMenu.Enable(True)
        self.selectAll.Enable(False)
        main.menuPicker.getAllMenu.Enable(False)
        main.currentItem = None
        main.showPreview(event)


    # Remove all items from renaming list
    def onSelectNone(self, event):
        total_items = self.ItemList.GetItemCount()
        # clear list
        self.Core.clearJoinedItems()
        # show deselected in picker
        for i in range(total_items):
            item = wx.ListItem()
            item.SetId(i)
            item.SetBackgroundColour(utils.BackgroundClr)
            item.SetTextColour(utils.TxtClr)
            self.ItemList.SetItem(item)
        # enable buttons
        self.selectAll.Enable(True)
        main.menuPicker.getAllMenu.Enable(True)
        self.selectNone.Enable(False)
        main.menuPicker.getNoneMenu.Enable(False)
        main.currentItem = None
        main.notebook.GetPage(3).clearErrors()
        main.setStatusMsg(_(u"All items were deselected"), 'eyes')
        main.showPreview(event)


    # Enable buttons (also called by errors class)
    def enableButtons(self):
        total_items = self.ItemList.GetItemCount()
        if self.Core.joinedItems != []:
            self.selectNone.Enable(True)
            main.menuPicker.getNoneMenu.Enable(False)
            if len(self.Core.joinedItems) == total_items:
                self.selectAll.Enable(False)
                main.menuPicker.getAllMenu.Enable(False)
            else:
                self.selectAll.Enable(True)
                main.menuPicker.getAllMenu.Enable(True)
        else:
            self.selectAll.Enable(True)
            main.menuPicker.getAllMenu.Enable(True)
            self.selectNone.Enable(False)
            main.menuPicker.getNoneMenu.Enable(False)


    # When setting walk checkbox from app menu
    def walkFromMenu(self):
        if self.walkIt.GetValue() == False:
            self.walkIt.SetValue(True)
        else:
            self.walkIt.SetValue(False)
        self.onRecursiveCheckbox(True)


    def onRecursiveCheckbox(self, event):
        if self.walkIt.GetValue():
            self.foldersOn.Enable(False)
            self.filesOn.Enable(False)
            self.walkDepth.Enable(True)
            self.staticText2.Enable(True)
        else:
            self.foldersOn.Enable(True)
            self.filesOn.Enable(True)
            self.walkDepth.Enable(False)
            self.staticText2.Enable(False)
        if event:
            self.refreshItems(event)


    # Enable display regexp options
    def regexOptions(self, event):
        if self.filterByRE.GetValue():
            self.ignoreCase.Enable(True)
            self.useLocale.Enable(True)
        else:
            self.ignoreCase.Enable(False)
            self.useLocale.Enable(False)
        if event:
            self.refreshItems(event)

