# -*- coding: utf-8 -*-

# Renaming Engine
# Takes names generated by the preview and applies changes to disc.
# Undoes renaming operations.

"""
Copyright (C) 2006-2010 ianaré sévi

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

ianare@gmail.com
"""

from __future__ import print_function
import utils
import os
import time
import codecs
import platform
import sys

import shutil1 # some earlier python versions have a bug in shutil
import classes

class Core():
    def __init__(self, MainWindow):
        global main
        main = MainWindow
        self.setup()


    # should be called before running to intialise
    def setup(self):
        self.prefs = main.prefs


    # refreshes display and main window during renaming operation
    def setDisplay(self, i):
        main.bottomWindow.display.EnsureVisible(i)
        main.bottomWindow.display.Refresh()
        main.Update()


    # show renaming errors
    def showRenameError(self, i, err, original, renamed):
        err = err.decode(sys.getfilesystemencoding())
        # set item as bad
        main.toRename[i][1][1] = None
        self.setDisplay(i)
        # show error message
        main.setStatusMsg(_("Renaming Failed : %s")%err, u'failed')
        utils.makeErrMsg(_(u"Error renaming:\n%s \n\nto:\n%s \n\nReason: %s")\
                         %(original[0],renamed[0],err),
                         _(u"Renaming Failed !!"))


    # renaming operation, with recursive error handling
    # returns True on success
    def rename(self, i, original, renamed, refresh_int):
        try:
            shutil1.move(original[0], renamed[0], True)
        except IOError as (errNumb, err):
            if main.debug:
                print("IOError : %s, %s"%(errNumb, err))
            # may need to create dirs first
            if errNumb == 2 and not os.path.exists(os.path.dirname(renamed[0])):
                try:
                    os.makedirs(os.path.dirname(renamed[0]))
                except OSError as (n, err):
                    self.showRenameError(i, err, original, renamed)
                    return 'makedirs'
                else:
                    return self.rename(i, original, renamed, refresh_int)
            else:
                self.showRenameError(i, err, original, renamed)
                return True
        except OSError as (errNumb, err):
            if main.debug:
                print("OSError : %s, %s"%(errNumb, err))
            # don't stop for a read-only error if the renamed item exists
            if not (errNumb == 30 and os.path.exists(renamed[0])):
                self.showRenameError(i, err, original, renamed)
                return True

        # set item as renamed
        main.toRename[i][1][1] = True

        # refresh based on user settings
        if i % refresh_int == 0:
            self.setDisplay(i)

        # renaming is successful
        return False


    # dupe checking, windows is case insensitive
    def compare(self, renamed, original):
        renamed = renamed[0]
        original = original[0]
        if platform.system() == 'Windows':
            renamed = renamed.lower()
            original = original.lower()
        if renamed != original:
            return True
        else:
            return False


    # after pre-renaming checks are completed, rename the list of items
    def renameItemList(self, event):
        # calculate refresh settings
        count = len(main.toRename)
        count = count / 25 + 1
        if count > int(self.prefs.get('renRefreshMin')):
            count = int(self.prefs.get('renRefreshMin'))

        # define here for faster processing
        def exists(item):
            return os.path.exists(item)
        def join(a,b,c):
            return unicode(os.path.join(a,b,c))

        warn = False
        if event != u'undo':
            path = main.picker.params.root

        # renaming operation:
        i = 0
        for original, renamed in main.toRename:
            if warn != 'duplicate_name' and self.compare(renamed, original) and\
              exists(renamed[0]):
                # set item as bad
                main.toRename[i][1][1] = None
                self.setDisplay(i)
                main.setStatusMsg(_(u"Duplicate name"), u'warn')
                msg = _(u"This name already exists:\n%s")%renamed[0]\
                    +_(u"\n\nI can make a sub-folder called 't_e_m_p' and do the renaming operation there.\nYou can then move everything back.\n\nGo ahead and do this?")
                title = _(u"Duplicate name")
                if utils.makeYesNoMsg(msg, title):
                    warn = 'duplicate_name'
                else:
                    error = True
                    break

            # set correct path if in dupe error mode
            if warn == 'duplicate_name':
                if not path.endswith(sep):
                    path = path+sep
                renamed[0] = renamed[0].replace(path,u'')
                renamed[0] = join(path, u't_e_m_p', renamed[0])

            # write name to undo
            self.originalFile.write(original[0] + u'\n')
            self.renamedFile.write(renamed[0] + u'\n')

            # the actual renaimg op
            error = self.rename(i, original, renamed, count)

            if error:
                break

            i += 1

            if main.debug:
                try:
                    print(original[0])
                    print(renamed[0])
                    print()
                except:
                    print('could not display name on this terminal')
                    print()

        return error, i


    # print out a list (for testing/debug)
    def printList(self, list):
        for item in list:
            print(item)


    # Write undo files first (safety first !), then attemp to perform
    # the renaming operation.
    def renameItems(self, event):
        itemsRenamed = 0
        error = False
        main.currentItem = None
        main.bottomWindow.display.mode = 'rename'
        utils.setBusy(True)
        main.picker.Panel.path.SetEditable(False)

        ##### BEGIN HEAVY TESTING ##############################################

        # adjust and sort items when recursively renaming folders
        if main.recursiveFolderOn and event != u'undo':
            main.setStatusMsg(_(u"Adjusting %s recursive paths, please wait ...")%len(main.toRename),u'wait')

            progressDialog = classes.ProgressDialog(main, self.prefs, main.items,
                             _(u"Adjusting %%% recursive paths, please wait ..."))

            if main.showTimes:
                t = time.time()

            # define these here for faster processing
            def osSplit(item):
                return os.path.split(item)
            def addSep(path):
                return os.sep + path + os.sep
            # order by path depth
            def recursiveFolderSort(x, y):
                x = x[1][0].count(os.sep)
                y = y[1][0].count(os.sep)
                return cmp(x, y)

            main.toRename.sort(recursiveFolderSort)
            sortedRename = main.toRename

            # make a list of all folders that will be processed
            foldersToAdjust = []
            for item in sortedRename:
                # only grab folders
                if not item[0][1]:
                    #oF = osSplit(item[0][0])[0] # original folder name
                    #nF = osSplit(item[1][0])[0] # new folder name
                    oF = item[0][0]
                    nF = item[1][0]
                    if oF is not nF:
                        print(oF, nF)
                        #print item[1][0].count(os.sep)
                        foldersToAdjust.append((oF, nF))
            sys.exit()

            # replace all occurences of folders in path
            for i in range(len(main.toRename)):

                print(foldersToAdjust)
                for f in foldersToAdjust:
                    oF = addSep(f[0]) # original folder name

                    # continue from this point for faster processing
                    if oF not in main.toRename[i][0][0]:
                        continue

                    nF = addSep(f[1]) # new folder name

                    print(oF, nF)
                    #print testing(i,oF,nF)

                    print(main.toRename[i][0][0])
                    print(main.toRename[i][0][0].replace(oF,nF,1))
                    print()

                    main.toRename[i][0][0] = main.toRename[i][0][0].replace(oF,nF,1)
                    main.toRename[i][1][0] = main.toRename[i][1][0].replace(oF,nF,1)

                #if not progressDialog.update(i):
                #    error = 'cancelled'
                #    break

            # make sure renaming will be in correct order !
            main.toRename.sort(recursiveFolderSort)

            progressDialog.destroy()

            if main.showTimes:
                print("%s items recursive adjust : %s"%(len(main.toRename), (time.time() - t)))

            #self.printList(foldersToAdjust)
            #self.printList(main.toRename)
            sys.exit()
            
        ##### END HEAVY TESTING ################################################

        if not error:
            main.setStatusMsg(_(u"Renaming in progress, please wait ..."),u'wait')

            # open undo files for writing
            try:
                self.originalFile = codecs.open(utils.stdPath(u'undo/original.bak'),'w',"utf-8")
                self.renamedFile = codecs.open(utils.stdPath(u'undo/renamed.bak'),'w',"utf-8")
            except IOError as (errno, strerror):
                msg = strerror + _(u"\nMake sure 'undo' directory exists and is read/write\n\nYou will not be able to undo!!\nDo you want to continue??")
                title = _(u"Problem with undo!")
                if not utils.makeYesNoMsg(msg, title):
                    error = 'cancelled'
                dlg.Destroy()

        # enough checking, DO IT !!
        if not error:
            if main.showTimes:
                t = time.time()
            # rename the item list
            error, itemsRenamed = self.renameItemList(event)

        # end of operations
        if not error:
            main.setStatusMsg(_(u"Renaming for %s items completed")
                              %itemsRenamed, u"complete")
            if self.prefs.get('reloadAfterRename') and event != u'undo':
                main.picker.refresh(event)
            else:
                main.picker.clearALL()
            if self.prefs.get(u'alwaysMakeLog'):
                main.saveAsText(False)
        elif error == 'cancelled':
            main.setStatusMsg(_(u"Renaming cancelled"), u'failed')

        utils.setBusy(False)
        self.setDisplay(len(main.toRename)-1)

        if main.showTimes:
            print("%s items renamed : %s"%(itemsRenamed, (time.time() - t)))

        # auto
        if main.autoModeLevel > 2 and not error:
            print(_(u"Renaming for %s items completed")%itemsRenamed)
            sys.exit()

        main.picker.refreshDirTree()

        main.bottomWindow.go.Enable(False)
        main.menuFile.GoMenu.Enable(False)
        if event != 'undo':
            main.bottomWindow.setUndoRedoType('undo')
        main.picker.Panel.path.SetEditable(True)


    # Grabs names from .bak files and reverts them to their originals.
    # Keep this separate from csv file functions for safety/stability
    def undoLastRename(self, event):
        original = []
        renamed = []
        utils.setBusy(True)
        try:
            originalFile = codecs.open(utils.stdPath(u'undo/original.bak'),'r',
                           'utf-8')
            renamedFile = codecs.open(utils.stdPath(u'undo/renamed.bak'),'r',
                          'utf-8')
            for line in originalFile:
                original.append([line.strip(),False])
            for line in renamedFile:
                renamed.append([line.strip(),1])

        except IOError as error:
            utils.makeErrMsg(_(u"%s\n\nUndo Failed !!")%error, _(u"Error"))
            pass
        else:
            def getName(x): return x[0]
            commonPrefix = os.path.commonprefix(map(getName,original)).rstrip(os.sep)
            if os.path.exists(commonPrefix):
                main.picker.Panel.path.SetValue(commonPrefix)
                main.picker.Panel.dirPicker.SetPath(commonPrefix)

            if not len(original) == 0:
                main.toRename = zip(renamed,original)#reverse order from original rename!
                main.currentItem = None
                main.displayResults()
                main.renameItems(u'undo')
                main.menuFile.SaveLog.Enable(False)
            else:
                main.setStatusMsg(_(u"Nothing to undo"), u'eyes')

        main.bottomWindow.setUndoRedoType('check')
        utils.setBusy(False)
